@@ -30,16 +30,28 @@ do {                                                                    \
 
 static
 bool parse_dim_for_one_arg(// input and output
+
+                           // so-far-seen named dimensions. Initially these are
+                           // <0 to indicate that they're unknown. As the
+                           // broadcasting rules determine the values of these,
+                           // the values are stored here (>= 0), and checked for
+                           // consistency
                            npy_intp* dims_named,
+
+                           // so-far-seen broadcasted dimensions. Initially
+                           // these are 1 to indicate that these are compatible
+                           // with anything. As non-1 values are seen, those are
+                           // stored here (> 1), and checked for consistency
                            npy_intp* dims_extra,
 
                            // input
-                           int Ndims_extra,
+                           const int Ndims_extra,
+                           const int Ndims_extra_inputs_only,
                            const char* arg_name,
-                           int Ndims_extra_var,
-                           const npy_intp* dims_want, int Ndims_want,
-                           const npy_intp* dims_var,  int Ndims_var,
-                           bool is_output)
+                           const int Ndims_extra_var,
+                           const npy_intp* dims_want, const int Ndims_want,
+                           const npy_intp* dims_var,  const int Ndims_var,
+                           const bool is_output)
 {
     // MAKE SURE THE PROTOTYPE DIMENSIONS MATCH (the trailing dimensions)
     //
@@ -99,31 +111,78 @@ bool parse_dim_for_one_arg(// input and output
     // This argument has Ndims_extra_var dimensions above the prototype (may be
     // <0 if there're implicit leading length-1 dimensions at the start). The
     // current dimensions to broadcast must match
-    //
-    // Also handle special broadcasting logic for outputs. The extra dimensions
-    // on the output must match the extra dimensions from the inputs EXACTLY.
-    // Some things could be reasonably supported, but it's all not very useful,
-    // and error-prone
-    if(is_output && Ndims_extra_var != Ndims_extra)
+
+    // outputs may be bigger than the inputs (this will result in multiple
+    // identical copies in each slice), but may not be smaller. I check that
+    // existing extra dimensions are sufficiently large. And then I check to
+    // make sure we have enough extra dimensions
+    if(is_output)
     {
-        PyErr_Format(PyExc_RuntimeError,
-                     "Outputs must match the broadcasted dimensions EXACTLY. '%s' has %d extra, broadcasted dimensions while the inputs have %d",
-                     arg_name,
-                     Ndims_extra_var, Ndims_extra);
-        return false;
+        for( int i_dim=-1;
+             i_dim >= -Ndims_extra_var;
+             i_dim--)
+        {
+            const int i_dim_var = i_dim - Ndims_want + Ndims_var;
+            // if we didn't get enough dimensions, use dim=1
+            const int dim_var = i_dim_var >= 0 ? dims_var[i_dim_var] : 1;
+
+            const int i_dim_extra = i_dim + Ndims_extra;
+
+            if(dim_var < dims_extra[i_dim_extra])
+            {
+                PyErr_Format(PyExc_RuntimeError,
+                             "Output '%s' dimension %d (broadcasted dimension %d) too small. Inputs have length %d but this output has length %d",
+                             arg_name,
+                             i_dim-Ndims_want, i_dim,
+                             (int)dims_extra[i_dim_extra],
+                             dim_var);
+                return false;
+            }
+        }
+
+        // I look through extra dimensions above what this output has to make
+        // sure that the output array is big-enough to hold all the output. I
+        // only care about the broadcasted slices defined by the input. Because
+        // I don't NEED to store all the duplicates created by the output-only
+        // broadcasting
+        for( int i_dim=-Ndims_extra_var-1;
+             i_dim >= -Ndims_extra_inputs_only;
+             i_dim--)
+        {
+            const int i_dim_extra = i_dim + Ndims_extra;
+
+            // What if this test passes, but a subsequent output increases
+            // dims_extra[i_dim_extra] so that this would have failed? That is
+            // OK. Extra dimensions in the outputs do not create new and
+            // different results, and I don't need to make sure I have room to
+            // store duplicates
+            if(dims_extra[i_dim_extra] > 1)
+            {
+                // This dimension was set, but this array has a DIFFERENT value
+                PyErr_Format(PyExc_RuntimeError,
+                             "Argument '%s' dimension %d (broadcasted dimension %d) is too small: this dimension of this output is too small to hold the broadcasted results of size %d",
+                             arg_name,
+                             i_dim-Ndims_want, i_dim,
+                             (int)dims_extra[i_dim_extra]);
+                return false;
+            }
+        }
     }
 
+
     for( int i_dim=-1;
          i_dim >= -Ndims_extra_var;
          i_dim--)
     {
-        int i_dim_var = i_dim - Ndims_want + Ndims_var;
+        const int i_dim_var = i_dim - Ndims_want + Ndims_var;
         // if we didn't get enough dimensions, use dim=1
-        int dim_var = i_dim_var >= 0 ? dims_var[i_dim_var] : 1;
+        const int dim_var = i_dim_var >= 0 ? dims_var[i_dim_var] : 1;
+
+        const int i_dim_extra = i_dim + Ndims_extra;
+
 
         if (dim_var != 1)
         {
-            int i_dim_extra = i_dim + Ndims_extra;
             if(i_dim_extra < 0)
             {
                 PyErr_Format(PyExc_RuntimeError,
@@ -134,20 +193,13 @@ bool parse_dim_for_one_arg(// input and output
                 return false;
             }
 
-            if(is_output && dims_extra[i_dim_extra] != dim_var)
-            {
-                PyErr_Format(PyExc_RuntimeError,
-                             "Outputs must match the broadcasted dimensions EXACTLY. '%s' dimension %d (broadcasted dimension %d) has length %d, while the inputs have %d",
-                             arg_name,
-                             i_dim-Ndims_want, i_dim,
-                             dim_var, dims_extra[i_dim_extra]);
-                return false;
-            }
-
+            // I have a new value for this dimension
             if( dims_extra[i_dim_extra] == 1)
+                // This dimension wasn't set yet; I set it
                 dims_extra[i_dim_extra] = dim_var;
             else if(dims_extra[i_dim_extra] != dim_var)
             {
+                // This dimension was set, but this array has a DIFFERENT value
                 PyErr_Format(PyExc_RuntimeError,
                              "Argument '%s' dimension %d (broadcasted dimension %d) mismatch. Previously saw length %d, but here have length %d",
                              arg_name,
